#!/usr/bin/env python

from twisted.python import log
from twisted.internet import defer, error

from tx11 import core, etx11, atoms, event
from tx11.proto import xproto, render


def find_format(screen, depth, visual):
    for d in screen.depths:
        if d.depth == depth:
            for v in d.visuals:
                if v.visual == visual:
                    return v.format

    raise Exception('failed to find an appropriate Render pictformat!')


def paint(conn, event, window, pid):
    conn.core.clearArea(False, window, 0, 0, 0, 0)

    for x in xrange(7):
        for y in xrange(5):
            rectangle = ((x + 1) * 24 + x * 64, (y + 1) * 24 + y * 64, 64, 64)
            color = (x * 65535 / 7, y * 65535 / 5, (x * y) * 65535 / 35,
                     65535)
            conn.render.fillRectangles(render.PictOp.Src, pid, color,
                                       [rectangle]).addErrback(log.err)


def enter(conn, event):
    print 'Enter (%d, %d)' % (event.event_x, event.event_y)

def leave(conn, event):
    print 'Leave (%d, %d)' % (event.event_x, event.event_y)

def buttonpress(conn, event):
    print 'Button %d down' % (event.detail,)
    conn.transport.loseConnection()


@defer.inlineCallbacks
def startup(reactor):
    d = defer.Deferred()
    handler = event.SimpleHandler()
    handler.addEventHandler(enter, xproto.EnterNotifyEvent)
    handler.addEventHandler(leave, xproto.LeaveNotifyEvent)
    handler.addEventHandler(buttonpress, xproto.ButtonPressEvent)
    def shutdown(reason):
        if reason.check(error.ConnectionDone):
            d.callback(None)
        else:
            d.errback(reason)
    try:
        conn = yield core.connect(
            reactor,
            extensions=[render.RenderExtension],
            shutdownHandler=shutdown, eventHandler=handler)
    except etx11.ConnectionDenied, e:
        print 'denied:', e.message.reason.tostring()
    except etx11.ConnectionFailed, e:
        print 'failed:', e.message.reason.tostring()
    except etx11.ExtensionMissing:
        print 'No render extension?'
    else:
        print 'Click in window to exit.'
        rootscreen = conn.setup.roots[0]
        root = rootscreen.root
        depth = rootscreen.root_depth
        visual = rootscreen.root_visual
        window = yield conn.generateId()
        pid = yield conn.generateId()

        white = rootscreen.white_pixel

        conn.core.createWindow(
            depth, window, root, 0, 0, 640, 480, 0,
            xproto.WindowClass.InputOutput, visual,
            backPixel=white, eventMask=(
                xproto.EventMask.ButtonPress |
                xproto.EventMask.EnterWindow |
                xproto.EventMask.LeaveWindow |
                xproto.EventMask.Exposure)).addErrback(log.err)

        reply = yield conn.render.queryPictFormats()
        format = find_format(reply.screens[0], depth, visual)

        handler.addEventHandler(lambda c, e: paint(c, e, window, pid),
                                xproto.ExposeEvent)

        name = 'Twisted X Python Binding Demo'
        conn.core.changeProperty(xproto.PropMode.Replace, window,
                                 atoms.XA_WM_NAME, atoms.XA_STRING, 8,
                                 name).addErrback(log.err)
        conn.render.createPicture(pid, window, format).addErrback(log.err)
        conn.core.mapWindow(window).addErrback(log.err)
        yield d


if __name__ == '__main__':
    from twisted.internet import reactor
    # This little dance means all our code runs from a deferred
    # callback (which means both sync and async exceptions make it to
    # the following errback) and with a running reactor (which
    # prevents the embarassing mistake of calling reactor.stop() from
    # a synchronously executing errback (which fails) and *then*
    # calling reactor.start() (which gets stuck)).
    d = defer.Deferred()
    d.addCallback(startup)
    d.addErrback(log.err)
    d.addBoth(lambda ignored: reactor.stop())
    reactor.callWhenRunning(d.callback, reactor)
    reactor.run()
