
"""Exceptions raised by tx11."""

import exceptions


class Exception(exceptions.Exception):
    pass


class ProtocolException(Exception):
    pass

class RemoteError(ProtocolException):
    """Base class for error replies from the server."""


class InitFailure(ProtocolException):
    pass

class ConnectionFailed(InitFailure):
    pass

class ConnectionDenied(InitFailure):
    pass

class ExtensionMissing(InitFailure):
    """Raised when a requested extension is missing.

    Currently the core does not depend on any extensions.
    This is only raised if user code requests additional extensions.

    The "missing" attribute is a sequence of missing extensions.
    """
    def __init__(self, missing):
        InitFailure.__init__(self)
        self.missing = missing


class XauthorityException(Exception):
    """Raised if there is something wrong with the Xauthority file."""


class NoMoreIds(Exception):
    """Raised from generateId if we do not know about any free ids."""
