
"""Some of xcb-util's render stuff.

This is very preliminary. Expect it to change incompatibly.
"""

from .proto import render


TEMPLATE_KEYS = frozenset([
        'id', 'type', 'depth', 'colormap',
        'red_shift', 'red_mask', 'green_shift', 'green_mask',
        'blue_shift', 'blue_mask', 'alpha_shift', 'alpha_mask',
        ])


STANDARD_ARGB_32 = dict(
    type=render.PictType.Direct, depth=32,
    red_shift=16, red_mask=0xff,
    green_shift=8, green_mask=0xff,
    blue_shift=0, blue_mask=0xff,
    alpha_shift=24, alpha_mask=0xff,
    )

STANDARD_RGB_24 = dict(
    type=render.PictType.Direct, depth=24,
    red_shift=16, red_mask=0xff,
    green_shift=8, green_mask=0xff,
    blue_shift=0, blue_mask=0xff,
    alpha_mask=0)

STANDARD_A_8 = dict(
    type=render.PictType.Direct, depth=8,
    red_mask=0, green_mask=0, blue_mask=0,
    alpha_shift=0, alpha_mask=0xff)

STANDARD_A_4 = dict(
    type=render.PictType.Direct, depth=8,
    red_mask=0, green_mask=0, blue_mask=0,
    alpha_shift=0, alpha_mask=0xf)

STANDARD_A_1 = dict(
    type=render.PictType.Direct, depth=8,
    red_mask=0, green_mask=0, blue_mask=0,
    alpha_shift=0, alpha_mask=1)


def find_visual_format(screen, depth, visual):
    for d in screen.depths:
        if depth is None or d.depth == depth:
            for v in d.visuals:
                if v.visual == visual:
                    return v.format
    raise LookupError()


def find_format(reply, template):
    # We could do this, but is there much point?
    #assert not set(template) - TEMPLATE_KEYS, 'bogus template'
    for format in reply.formats:
        for attr in ['id', 'type', 'depth', 'colormap']:
            if attr in template and template[attr] != getattr(format, attr):
                break
        else:
            for attr in [
                'red_mask', 'green_mask', 'blue_mask', 'alpha_mask',
                'red_shift', 'green_shift', 'blue_shift', 'alpha_shift']:
                val = template.get(attr)
                if val is not None and getattr(format.direct, attr) != val:
                    break
            else:
                return format
    raise LookupError()
